<?php

namespace Drupal\marketo_ma_user\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\marketo_ma\MarketoFieldDefinition;
use Drupal\marketo_ma\Service\MarketoMaServiceInterface;
use Drupal\marketo_ma_user\Service\MarketoMaUserServiceInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Marketo user settings form.
 */
class MarketoMaUserSettings extends ConfigFormBase {

  /**
   * The Marketo MA API client.
   *
   * @var \Drupal\marketo_ma\Service\MarketoMaServiceInterface
   */
  protected $marketoMaService;

  /**
   * The Marketo MA API client.
   *
   * @var \Drupal\marketo_ma_user\Service\MarketoMaUserServiceInterface
   */
  protected $marketoMaUserService;

  /**
   * The entity field manager.
   *
   * @var \Drupal\Core\Entity\EntityFieldManagerInterface
   */
  protected $entityFieldManager;

  /**
   * Constructs a \Drupal\marketo_ma\Form\MarketoMaSettings object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   * @param \Drupal\marketo_ma\Service\MarketoMaServiceInterface $marketo_ma_service
   *   The Marketo MA API client.
   * @param \Drupal\marketo_ma_user\Service\MarketoMaUserServiceInterface $marketo_ma_user_service
   *   The Marketo MA API client.
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface $entity_field_manager
   *   The entity field manager.
   */
  public function __construct(ConfigFactoryInterface $config_factory, MarketoMaServiceInterface $marketo_ma_service, MarketoMaUserServiceInterface $marketo_ma_user_service, EntityFieldManagerInterface $entity_field_manager) {
    parent::__construct($config_factory);
    $this->marketoMaService = $marketo_ma_service;
    $this->marketoMaUserService = $marketo_ma_user_service;
    $this->entityFieldManager = $entity_field_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('marketo_ma'),
      $container->get('marketo_ma.user'),
      $container->get('entity_field.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [MarketoMaUserServiceInterface::MARKETO_MA_USER_CONFIG_NAME];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'marketo_ma_user_settings';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    /** @var array $form */
    $form = parent::buildForm($form, $form_state);

    // Get the configuration.
    $config = $this->config(MarketoMaUserServiceInterface::MARKETO_MA_USER_CONFIG_NAME);

    $form['group_events'] = [
      '#title' => $this->t('Events'),
      '#type' => 'fieldset',
    ];
    $form['group_mappings'] = [
      '#title' => $this->t('User field mapping'),
      '#type' => 'fieldset',
    ];
    $form['group_activities'] = [
      '#title' => $this->t('Enabled Activities'),
      '#type' => 'fieldset',
      '#description' => $this->t('Note: Activities are retrieved by type in batches of 10 types. Each multiple of 10 activity types will require another API request and will afffect their performance of activity pages.'),
    ];
    // Add the role tracking settings.
    $form['group_events']['events'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Trigger a lead update on the following events:'),
      '#default_value' => $config->get('events'),
      '#options' => [
        'login' => $this->t('User login'),
        'create' => $this->t('User registration / creation'),
        'update' => $this->t('User update'),
      ],
    ];

    $form['group_mappings']['mapping'] = [
      '#type' => 'table',
      '#header' => [
        'title' => $this->t('User Field'),
        'mapping' => $this->t('Marketo Field'),
      ],
      '#empty' => $this->t('There are no user fields available for mapping.'),
    ];

    // Get mappings from config.
    $mapping = $config->get('mapping');
    // Get enabled marketo fields.
    $marketo_field_options = ['' => $this->t('None')] + $this->getMarketoFields();
    // Add an mapping select field for each user field.
    foreach ($this->getUserFields() as $field_name => $label) {
      $form['group_mappings']['mapping'][$field_name] = [
        'title' => ['#markup' => $label],
        'mapping' => [
          '#type' => 'select',
          '#title' => $this->t('Select mapped component'),
          '#title_display' => 'hidden',
          '#options' => $marketo_field_options,
          '#default_value' => isset($mapping[$field_name]) ? $mapping[$field_name] : FALSE,
        ],
      ];
    }

    $form['group_activities']['enabled_activities'] = [
      '#type' => 'tableselect',
      '#title' => $this->t('Enabled Activities'),
      '#header' => [
        $this->t('ID'),
        $this->t('Name'),
        $this->t('Description'),
        $this->t('Primary attribute name'),
      ],
      '#options' => $this->getActivityOptions(),
      '#empty' => $this->t('No activity types, try retrieving from marketo.'),
      '#prefix' => '<div id="marketo-enabled-activities-wrapper">',
      '#suffix' => '</div>',
      '#default_value' => $config->get('enabled_activities'),
    ];
    // Add the ajax button that get's fields from the marketo API.
    $form['group_activities']['activity_api_retrieve_activities'] = [
      '#type' => 'button',
      '#value' => $this->t('Fetch from Marketo'),
      '#disabled' => !$this->marketoMaService->apiClientCanConnect(),
      '#ajax' => [
        'callback' => [$this, 'retrieveApiActivities'],
        'event' => 'mouseup',
        'wrapper' => 'marketo-enabled-activities-wrapper',
        'progress' => [
          'type' => 'throbber',
          'message' => $this->t('Retrieving fields from Marketo...'),
        ],
      ],
    ];

    // Add the validation and submit callbacks.
    $form['#validate'][] = [$this, 'validateForm'];
    $form['#submit'][] = [$this, 'submitForm'];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    parent::submitForm($form, $form_state);

    // Get the mapping values.
    $mapping = array_map(function ($form_value) {
      return $form_value['mapping'];
    }, $form_state->getValue('mapping'));

    $this->config(MarketoMaUserServiceInterface::MARKETO_MA_USER_CONFIG_NAME)
      ->set('events', array_filter($form_state->getValue('events')))
      ->set('mapping', array_filter($mapping))
      ->set('enabled_activities', array_filter($form_state->getValue('enabled_activities')))
      ->save();
  }

  /**
   * Connects to Marketo and retrieves the API activity types.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @return array
   *   The form element to replace in the ajax wrapper setting.
   */
  public function retrieveApiActivities(array &$form, FormStateInterface $form_state) {
    $this->marketoMaUserService->getMarketoActivities(TRUE);
    $form['group_activities']['enabled_activities']['#options'] = $this->getActivityOptions();
    return $form['group_activities']['enabled_activities'];
  }

  /**
   * Build the options list for the activities tableselect.
   *
   * @return string[][]
   *   Table column values.
   */
  private function getActivityOptions() {
    return array_map(function ($item) {
      return [
        $this->t(':value', [':value' => $item->id()]),
        $this->t(':value', [':value' => $item->getName()]),
        $this->t(':value', [':value' => $item->getDescription()]),
        $this->t(':value', [':value' => $item->getPrimaryAttributeName()]),
      ];
    }, $this->marketoMaUserService->getMarketoActivities());
  }

  /**
   * Returns the available marketo field.
   *
   * @return string[]
   *   Marketo field labels keyed by machine name.
   */
  protected function getMarketoFields() {
    return array_map(function (MarketoFieldDefinition $field) {
      return $field->getDisplayName();
    }, $this->marketoMaService->getEnabledFields());
  }

  /**
   * Returns all available user form field.
   *
   * @return string[]
   *   Additional field labels keyed by machine name.
   */
  protected function getUserFields() {
    $fields = $this->entityFieldManager->getFieldDefinitions('user', 'user');
    return array_map(function (FieldDefinitionInterface $field_definition) {
      return $field_definition->getLabel();
    }, $fields);
  }

}
